/*
 * Copyright (c) 2008 Christian W. Guenther (christian@deckfour.org)
 * Copyright (c) 2013 F. Mannhardt (f.mannhardt@tue.nl)
 * 
 * LICENSE:
 * 
 * This code is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 3 of the License, or (at your option) any
 * later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */
package org.processmining.xeslite.lite.factory;

import java.util.Set;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.id.XID;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeMap;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.XVisitor;
import org.deckfour.xes.util.XAttributeUtils;
import org.processmining.log.utils.XUtils;
import org.processmining.xeslite.common.XSeqIDFactory;

import com.google.common.primitives.Longs;

/**
 * @author F. Mannhardt
 * 
 */
class XEventLiteImpl implements XEvent {

	private long id;

	private XAttributeMap attributes;

	public XEventLiteImpl() {
		this(XSeqIDFactory.instance().nextId(), new XAttributeMapLiteImpl());
	}

	public XEventLiteImpl(long id) {
		this(id, new XAttributeMapLiteImpl());
	}

	public XEventLiteImpl(XAttributeMap attributes) {
		this(XSeqIDFactory.instance().nextId(), attributes);
	}

	public XEventLiteImpl(long id, XAttributeMap attributes) {
		this.id = id;
		this.attributes = attributes;
	}

	public XAttributeMap getAttributes() {
		return attributes;
	}

	public void setAttributes(XAttributeMap attributes) {
		this.attributes = attributes;
	}

	public boolean hasAttributes() {
		return !attributes.isEmpty();
	}

	public Set<XExtension> getExtensions() {
		return XAttributeUtils.extractExtensions(attributes);
	}

	@Override
	public Object clone() {
		XEventLiteImpl clone = new XEventLiteImpl();
		clone.id = XSeqIDFactory.instance().nextId();
		clone.attributes = (XAttributeMap) attributes.clone();
		return clone;
	}

	@Override
	public boolean equals(Object o) {
		if (o instanceof XEventLiteImpl) {
			return ((XEventLiteImpl) o).id == id;
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return Longs.hashCode(id);
	}

	public XID getID() {
		return new XID(id, id);
	}

	/*
	 * Taken from OpenXES
	 * 
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.deckfour.xes.model.XEvent#accept(org.deckfour.xes.model.XVisitor,
	 * org.deckfour.xes.model.XTrace)
	 */
	public void accept(XVisitor visitor, XTrace trace) {
		visitor.visitEventPre(this, trace);
		for (XAttribute attribute : getAttributes().values()) {
			attribute.accept(visitor, this);
		}
		visitor.visitEventPost(this, trace);
	}
	
	@Override
	public String toString() {
		return XUtils.stringifyEvent(this);
	}

}